"""Top-level package for {{ cookiecutter.project_name }}."""

from importlib.metadata import metadata

from {{ cookiecutter.package_name }}.sub_package_1.my_class_1 import MyClass1 as MyClass1
from {{ cookiecutter.package_name }}.sub_package_2.my_class_2 import MyClass2 as MyClass2
from {{ cookiecutter.package_name }}.sub_package_2.my_class_3 import MyClass3 as MyClass3


infos = metadata(__name__)
__version__ = infos["Version"]
__author__ = "{{ cookiecutter.full_name }}"
__email__ = "{{ cookiecutter.email }}"



#!/usr/bin/env python
from pathlib import Path

PROJECT_DIRECTORY = Path.cwd()

if __name__ == "__main__":
    if "no" in "{{ cookiecutter.command_line_interface|lower }}":
        (PROJECT_DIRECTORY / "{{ cookiecutter.package_name }}" / "cli.py").unlink()

    if not {{cookiecutter.dockerfile}}:
        (PROJECT_DIRECTORY / "Dockerfile").unlink()



from cookiecutter.utils import simple_filter


@simple_filter
def split_last(value):
    """
    Parameters
    ----------
    value: :class:`str`
        Firstname Lastname

    Returns
    -------
    :class:`dict`
        First and last name in a dict.

    Examples
    --------

    >>> split_last('Fabien Paul Cédric Sébastien Mathieu')
    {'left': 'Fabien Paul Cédric Sébastien', 'right': 'Mathieu'}
    """
    left, sep, right = value.rpartition(" ")
    return {"left": left, "right": right}



import os
import click


@click.command()
@click.version_option(message="Package Helper 3, version %(version)s")
def main() -> None:  # pragma: no cover
    """
    Package Helper 3 command that creates a UV-powered Python project.

    Cf https://balouf.github.io/package-helper-3/
    """
    cwd = os.path.dirname(__file__)
    package_dir = os.path.abspath(os.path.join(cwd, ".."))
    os.system(f"cookiecutter {package_dir}")



# -*- coding: utf-8 -*-
"""
Copyright {{ cookiecutter.full_name }}
{{ cookiecutter.email }}

This file is part of {{ cookiecutter.project_name }}.
"""


class MyClass2:
    """A whatever-you-are-doing.

    Parameters
    ----------
    a : float
        The `a` of the system.
    b : float
        The `b` of the system.

    Examples
    --------
        >>> my_object = MyClass2(a = 5, b = 3)
    """

    def __init__(self, a: float, b: float):
        self.a = a
        self.b = b

    def addition(self) -> float:
        """Add `a` and `b`.

        Returns
        -------
        Float
            The sum of `a` and `b`.

        Examples
        --------
            >>> my_object = MyClass2(a=5, b=3)
            >>> my_object.addition()
            8
        """
        return self.a + self.b



def foo() -> str:
    """
    Summary line.

    Extended description of function.

    Returns
    -------
    :class:`str`
        Description of return value

    Examples
    --------

    >>> foo()
    'foo'
    """

    return "foo"


def fibo(n):
    """

    Parameters
    ----------
    n: :class:`int`
        The sequence number

    Returns
    -------
    :class:`int`

    Examples
    --------

    >>> fibo(3)
    2
    >>> fibo(30)
    832040
    """
    return 1 if n < 3 else fibo(n - 1) + fibo(n - 2)


if __name__ == "__main__":  # pragma: no cover
    print(foo())



from importlib.metadata import metadata

infos = metadata(__name__)
__version__ = infos["Version"]

# Parse authors from Author-email (may contain multiple entries)
author_email = infos.get("Author-email")
authors = []

if author_email:
    # Split entries by commas (handles multiple authors)
    entries = [e.strip() for e in author_email.split(",")]
    for entry in entries:
        # Split each entry into name and email
        if "<" in entry and ">" in entry:
            name, email = entry.split("<", 1)
            email = email.strip(">").strip()
            authors.append({"name": name.strip(), "email": email})
        else:
            authors.append({"name": entry, "email": None})

# Expose as needed (e.g., first author)
__author__ = authors[0]["name"] if authors else "Unknown"
__email__ = authors[0]["email"] if authors else None



import os
import sys

sys.path.insert(0, os.path.abspath(".."))

import {{ cookiecutter.package_name }}

# -- General configuration ---------------------------------------------

# Add extensions here. See https://www.sphinx-doc.org/en/master/usage/extensions/
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.viewcode",
    "sphinx.ext.napoleon",
    "sphinx.ext.intersphinx",
    "sphinx.ext.imgconverter",
    "nbsphinx",
    "IPython.sphinxext.ipython_console_highlighting",
    "myst_parser",
    "sphinx_copybutton",
]

# Smart code copy: exclude prompts and output markers
copybutton_exclude = ".linenos, .gp, .go"

# MyST-Parser configuration
myst_enable_extensions = ["linkify", "dollarmath", "colon_fence"]
myst_heading_anchors = 3
myst_links_external_new_tab = True

# Intersphinx: cross-reference other projects' documentation.
# Add or remove mappings depending on which libraries your project uses.
intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "sklearn": ("https://scikit-learn.org/stable", None),
    "ipython": ("https://ipython.readthedocs.io/en/stable/", None),
    "numba": ("https://numba.readthedocs.io/en/stable/", None),
    "numpy": ("https://numpy.org/doc/stable/", None),
    "scipy": ("https://docs.scipy.org/doc/scipy/", None),
    "matplotlib": ("https://matplotlib.org/stable/", None),
}

templates_path = ["_templates"]
source_suffix = [".rst", ".md"]
master_doc = "index"

# Project information
project = "{{ cookiecutter.project_name }}"
copyright = "{% now 'local', '%Y' %}, {{ cookiecutter.full_name }}"
author = "{{ cookiecutter.full_name }}"

version = {{ cookiecutter.package_name }}.__version__
release = {{ cookiecutter.package_name }}.__version__

language = "en"
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]
pygments_style = "sphinx"

# -- Options for HTML output -------------------------------------------

html_theme = "pydata_sphinx_theme"
html_theme_options = {
    "icon_links": [
        {
            "name": "GitHub",
            "url": "https://github.com/{{ cookiecutter.github_username }}/{{ cookiecutter.project_slug }}",
            "icon": "fa-brands fa-github",
        },
    ],
    "header_links_before_dropdown": 5,
    "show_nav_level": 2,
    "show_toc_level": 2,
    "navigation_depth": 2,
}

# Uncomment to add custom static files (CSS, JS):
# html_static_path = ["_static"]

# -- Options for HTMLHelp output ---------------------------------------

htmlhelp_basename = "{{ cookiecutter.package_name }}doc"

# -- Options for LaTeX output ------------------------------------------

# Uncomment and adjust for LaTeX/PDF builds:
# latex_elements = {
#     "papersize": "a4paper",
#     "pointsize": "10pt",
# }

latex_documents = [
    (
        master_doc,
        "{{ cookiecutter.package_name }}.tex",
        "{{ cookiecutter.project_name }} Documentation",
        "{{ cookiecutter.full_name }}",
        "manual",
    ),
]

# -- Options for manual page output ------------------------------------

man_pages = [
    (
        master_doc,
        "{{ cookiecutter.package_name }}",
        "{{ cookiecutter.project_name }} Documentation",
        [author],
        1,
    ),
]

# -- Options for Texinfo output ----------------------------------------

texinfo_documents = [
    (
        master_doc,
        "{{ cookiecutter.package_name }}",
        "{{ cookiecutter.project_name }} Documentation",
        author,
        "{{ cookiecutter.package_name }}",
        "One line description of project.",
        "Miscellaneous",
    ),
]



# -*- coding: utf-8 -*-
"""
Copyright {{ cookiecutter.full_name }}
{{ cookiecutter.email }}

This file is part of {{ cookiecutter.project_name }}.
"""
from {{ cookiecutter.package_name }}.sub_package_2.my_class_2 import MyClass2


class MyClass1:
    """A whatever-you-are-doing.

    Parameters
    ----------
    a : :class:`float`
        The `a` of the system. Must be non-negative.
    b : :class:`float`
        The `b` of the system.

    Attributes
    ----------
    my_string : :class:`str`
        A nice string.

    Raises
    ------
    ValueError
        If `a` is negative.

    Notes
    -----
    Document the :meth:`__init__` method in the docstring of the class itself,
    because the docstring of the :meth:`__init__` method does not appear in the
    documentation.

    * Refer to a class this way: :class:`MyClass2` (except as a type indication, cf. :meth:`update_b_from_class_2`).
    * Refer to a method this way: :meth:`addition`.
    * Refer to a method in another class: :meth:`MyClass2.addition`.
    * Refer to an attribute this way: :attr:`my_string`.
    * Refer to a property this way: :attr:`a_square`.
    * Refer to a parameter or variable this way: `a`.

    Examples
    --------
    >>> my_object = MyClass1(a=5, b=3)
    """

    #: This is a nice constant.
    A_NICE_CONSTANT = 42
    #:
    A_VERY_NICE_CONSTANT = 51

    def __init__(self, a: float, b: float):
        if a < 0:
            raise ValueError('Expected non-negative a, got: ', a)
        self.a = a
        self.b = b
        self.my_string = 'a = %s and b = %s' % (a, b)

    def __repr__(self) -> str:
        return 'MyClass1(a=%r, b=%r)' % (self.a, self.b)

    def __str__(self) -> str:
        return '(a, b) = %s, %s' % (self.a, self.b)

    def divide_a_by_c_and_add_d(self, c: float, d: float) -> float:
        """Divide `a` by something and add something else.

        Parameters
        ----------
        c : Number
            A non-zero number. You can say many things about this parameter
            in several indented lines, like this.
        d : Number
            A beautiful number.

        Returns
        -------
        Number
            The result of `a / c + d`.

        Raises
        ------
        ZeroDivisionError
            If `c` = 0.

        Notes
        -----
        This function gives an example of documentation with typical features.

        Examples
        --------
        We can write some text to explain the following example:

            >>> my_object = MyClass1(a=5, b=3)
            >>> my_object.divide_a_by_c_and_add_d(c=2, d=10)
            12.5

        And we can explain a second example here:

            >>> my_object = MyClass1(a=5, b=3)
            >>> my_object.divide_a_by_c_and_add_d(c=2, d=20)
            22.5
        """
        return self.a / c + d

    def addition(self) -> float:
        """Add `a` and `b`.

        Returns
        -------
        Number
            The sum of `a` and `b`.

        Examples
        --------
            >>> my_object = MyClass1(a=5, b=3)
            >>> my_object.addition()
            8
        """
        return MyClass2(self.a, self.b).addition()

    @property
    def a_square(self) -> float:
        """The square of `a`."""
        return self.a ** 2

    def update_b_from_class_2(self, object_of_class_2):
        """Update `b` from a :class:`MyClass2` object.

        Parameters
        ----------
        object_of_class_2 : MyClass2
            An object from the other class. The purpose of this function is essentially to show how to document when
            an argument is an object of another class.

            N.B.: for the type of an argument, you can enter only the name of the class, e.g. ``MyClass2``.
            However, in the rest of the documentation, you must use the full syntax, like ``:class:`MyClass2```.

        Examples
        --------
            >>> my_object = MyClass1(a=5, b=3)
            >>> my_object.update_b_from_class_2(MyClass2(42, 51))
            >>> my_object.b
            51
        """
        self.b = object_of_class_2.b

    # noinspection PyProtectedMember
    def _secret_function(self) -> float:
        """Difference between `a` and `b`.

        Returns
        -------
        Number
            The result of `a` - `b`.

        Notes
        -----
        Since the name of this function starts with _, it does not appear in
        the Sphinx documentation (by default).

        Examples
        --------
            >>> my_object = MyClass1(a=5, b=3)
            >>> my_object._secret_function()
            2
        """
        return self.a - self.b
